import 'package:cross_file/cross_file.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:flutterx_drop_view/flutterx_drop_view.dart';
import 'package:flutterx_live_data/flutterx_live_data.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

class FlutterxDropView extends StatelessWidget {
  final DragOperation dragOperation;
  final SystemMouseCursor? cursor;
  final Set<String> mimeTypes;
  final void Function(MimeTypeNotMatchException error)? onMimeTypeNotMatch;
  final MutableLiveData<XFile?>? file;
  final LiveList<XFile>? files;
  final Widget Function(BuildContext context, LiveData<bool> dragging) builder;

  const FlutterxDropView.single({
    Key? key,
    this.dragOperation = DragOperation.copy,
    this.cursor,
    this.mimeTypes = const {},
    this.onMimeTypeNotMatch,
    required this.file,
    required this.builder,
  })  : files = null,
        super(key: key);

  const FlutterxDropView.multiple({
    Key? key,
    this.dragOperation = DragOperation.copy,
    this.cursor,
    this.mimeTypes = const {},
    this.onMimeTypeNotMatch,
    required this.files,
    required this.builder,
  })  : file = null,
        super(key: key);

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties
      ..add(EnumProperty<DragOperation>('dragOperation', dragOperation))
      ..add(DiagnosticsProperty<SystemMouseCursor?>('cursor', cursor))
      ..add(IterableProperty<String>('mimeTypes', mimeTypes))
      ..add(ObjectFlagProperty<void Function(MimeTypeNotMatchException error)?>.has(
          'onMimeTypeNotMatch', onMimeTypeNotMatch))
      ..add(DiagnosticsProperty<MutableLiveData<XFile?>?>('file', file))
      ..add(IterableProperty<XFile>('files', files))
      ..add(ObjectFlagProperty<Widget Function(BuildContext context, LiveData<bool> dragging)>.has('builder', builder));
  }

  @override
  Widget build(BuildContext context) => RawDropView(
      dragOperation: dragOperation,
      cursor: cursor,
      mimeTypes: mimeTypes,
      throwOnMimeTypeNotMatch: onMimeTypeNotMatch != null,
      multiple: files != null,
      onDrop: _handleDrop,
      builder: builder);

  void _handleDrop(Stream<XFile> files) => files.toList().then(_handleFiles,
      onError: (error) => error is MimeTypeNotMatchException ? onMimeTypeNotMatch?.call(error) : null);

  void _handleFiles(List<XFile> data) {
    files?.setTo(data);
    file?.value = data.iterator.let((iterator) => iterator.moveNext() ? iterator.current : null);
  }

  static void pickFiles({Set<String> mimeTypes = const {}, required LiveList<XFile> files, Function? onError}) =>
      RawDropView.pickFiles(mimeTypes: mimeTypes, multiple: true).toList().then(files.setTo, onError: onError);

  static void pickFile({Set<String> mimeTypes = const {}, required MutableLiveData<XFile?> file, Function? onError}) =>
      RawDropView.pickFiles(mimeTypes: mimeTypes).first.then((result) => file.value = result, onError: onError);
}
